@@ -45,10 +45,10 @@ extern "C" {
  * usually more strict because you're not really supposed to dump/load and
  * upgrade at the same time.
  */
-#define REV_LEVEL	805		/* This version */
+#define REV_LEVEL	811		/* This version */
 #define REV_MIN		591		/* Oldest compatible database */
 #define EXPORT_REV_MIN	760		/* Oldest compatible export files */
-#define LIBCITADEL_MIN	805		/* Minimum required version of libcitadel */
+#define LIBCITADEL_MIN	811		/* Minimum required version of libcitadel */
 
 #define SERVER_TYPE 0			/* zero for stock Citadel; other developers please
 					   obtain SERVER_TYPE codes for your implementations */
@@ -77,11 +77,6 @@ extern "C" {
  */
 typedef struct ExpirePolicy ExpirePolicy;
 struct ExpirePolicy {
-/*
-#include "datadefinitions.h"
-#include "dtds/expirepolicy-defs.h"
-#include "undef_data.h"
-*/
 	int expire_mode;
 	int expire_value;
 };
@@ -99,11 +94,6 @@ struct ExpirePolicy {
  */
 typedef struct march march;
 struct march {
-/*
-#include "datadefinitions.h"
-#include "dtds/march-defs.h"
-#include "undef_data.h"
-*/
 	struct march *next;
 	char march_name[ROOMNAMELEN];
 	unsigned int march_flags;
@@ -129,11 +119,6 @@ struct march {
  */
 typedef struct ctdluser ctdluser;
 struct ctdluser {			/* User record                      */
-/*
-#include "datadefinitions.h"
-#include "dtds/user-defs.h"
-#include "undef_data.h"
-*/
 	int version;			/* Cit vers. which created this rec  */
 	uid_t uid;			/* Associate with a unix account?    */
 	char password[32];		/* password                          */
@@ -159,11 +144,6 @@ struct ctdluser {			/* User record                      */
  */
 typedef struct ctdlroom ctdlroom;
 struct ctdlroom {
-/*
-#include "datadefinitions.h"
-#include "dtds/room-defs.h"
-#include "undef_data.h"
-*/
  	char QRname[ROOMNAMELEN];	/* Name of room                     */
  	char QRpasswd[10];		/* Only valid if it's a private rm  */
  	long QRroomaide;		/* User number of room aide         */
@@ -206,11 +186,6 @@ struct ctdlroom {
  */
 typedef struct floor floor;
 struct floor {
-/*
-#include "datadefinitions.h"
-#include "dtds/floor-defs.h"
-#include "undef_data.h"
-*/
 	unsigned short f_flags;		/* flags */
 	char f_name[256];		/* name of floor */
 	int f_ref_count;		/* reference count */
@@ -240,11 +215,6 @@ struct floor {
 #define	NEXT_KEY	15
 #define STOP_KEY	3
 
-/* server exit codes */
-#define EXIT_NORMAL	0		/* server terminated normally */
-					/* 1 through 63 reserved for signals */
-#define EXIT_NULL	64		/* EOF on server command input */
-
 /* citadel.rc stuff */
 #define RC_NO		0		/* always no */
 #define RC_YES		1		/* always yes */
@@ -262,6 +232,9 @@ enum {
 #define GF_SKIP		1		/* <;S>kip floor mode */
 #define GF_ZAP		2		/* <;Z>ap floor mode */
 
+/* number of items which may be handled by the CONF command */
+#define NUM_CONFIGS 70
+
 /*
  * MIME types used in Citadel for configuration stuff
  */
