@@ -1,22 +1,21 @@
 /*
  * DNS lookup for SMTP sender
  *
+ * Copyright (c) 1987-2011 by the citadel.org team
  *
- * Copyright (c) 1987-2009 by the citadel.org team
+ * This program is open source software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
+ * (at your option) any later version.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 3 of the License, or
- *  (at your option) any later version.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 #include "sysdep.h"
@@ -25,6 +24,7 @@
 #include <string.h>
 #include <netinet/in.h>
 #include <stdio.h>
+#include <syslog.h>
 
 #ifdef HAVE_RESOLV_H
 #include <arpa/nameser.h>
@@ -161,7 +161,7 @@ int getmx(char *mxbuf, char *dest) {
 	
 		for (qdcount = ntohs(answer.header.qdcount); qdcount--; ptr += ret + QFIXEDSZ) {
 			if ((ret = dn_skipname(ptr, endptr)) < 0) {
-				CtdlLogPrintf(CTDL_DEBUG, "dn_skipname error\n");
+				syslog(LOG_DEBUG, "dn_skipname error\n");
 				return(0);
 			}
 		}
