@@ -24,10 +24,10 @@ struct SpoolControl {
 };
 
 
-void network_spoolout_room(char *room_to_spool, 		       
+void network_spoolout_room(RoomProcList *room_to_spool, 		       
 			   char *working_ignetcfg,
 			   NetMap *the_netmap);
-void network_do_spoolin(char *working_ignetcfg, NetMap *the_netmap, int *netmap_changed);
+void network_do_spoolin(char *working_ignetcfg, NetMap **the_netmap, int *netmap_changed);
 void network_consolidate_spoolout(char *working_ignetcfg, NetMap *the_netmap);
 void free_spoolcontrol_struct(SpoolControl **scc);
 int writenfree_spoolcontrol_file(SpoolControl **scc, char *filename);
