@@ -2,21 +2,15 @@
  * A server-side module for Citadel which supports address book information
  * using the standard vCard format.
  * 
- * Copyright (c) 1999-2009 by the citadel.org team
+ * Copyright (c) 1999-2012 by the citadel.org team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
- * (at your option) any later version.
+ * This program is open source software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 3.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 /*
@@ -91,7 +85,7 @@ void set_mm_valid(void) {
  * Extract Internet e-mail addresses from a message containing a vCard, and
  * perform a callback for any found.
  */
-void vcard_extract_internet_addresses(struct CtdlMessage *msg, void (*callback)(char *, char *) ) {
+void vcard_extract_internet_addresses(struct CtdlMessage *msg, int (*callback)(char *, char *) ) {
 	struct vCard *v;
 	char *s;
 	char *k;
@@ -132,33 +126,55 @@ void vcard_extract_internet_addresses(struct CtdlMessage *msg, void (*callback)(
 
 	vcard_free(v);
 }
-
-
+///TODO: gettext!
+#define _(a) a
 /*
  * Callback for vcard_add_to_directory()
  * (Lotsa ugly nested callbacks.  Oh well.)
  */
-void vcard_directory_add_user(char *internet_addr, char *citadel_addr) {
+int vcard_directory_add_user(char *internet_addr, char *citadel_addr) {
+	struct CitContext *CCC = CC;
 	char buf[SIZ];
 
 	/* We have to validate that we're not stepping on someone else's
 	 * email address ... but only if we're logged in.  Otherwise it's
 	 * probably just the networker or something.
 	 */
-	if (CC->logged_in) {
-		syslog(LOG_DEBUG, "Checking for <%s>...\n", internet_addr);
+	if (CCC->logged_in) {
+		syslog(LOG_DEBUG, "Checking for <%s>...", internet_addr);
 		if (CtdlDirectoryLookup(buf, internet_addr, sizeof buf) == 0) {
 			if (strcasecmp(buf, citadel_addr)) {
 				/* This address belongs to someone else.
 				 * Bail out silently without saving.
 				 */
-				syslog(LOG_DEBUG, "DOOP!\n");
-				return;
+				syslog(LOG_DEBUG, "DOOP!");
+				
+				StrBufAppendPrintf(CCC->StatusMessage, "\n%d|", ERROR+ALREADY_EXISTS);
+				StrBufAppendBufPlain(CCC->StatusMessage, internet_addr, -1, 0);
+				StrBufAppendBufPlain(CCC->StatusMessage, HKEY("|"), 0);
+				StrBufAppendBufPlain(CCC->StatusMessage, _("Unable to add this email address again."), -1, 0);
+				StrBufAppendBufPlain(CCC->StatusMessage, HKEY("\n"), 0);
+				return 0;
 			}
 		}
 	}
-	syslog(LOG_INFO, "Adding %s (%s) to directory\n", citadel_addr, internet_addr);
-	CtdlDirectoryAddUser(internet_addr, citadel_addr);
+	syslog(LOG_INFO, "Adding %s (%s) to directory", citadel_addr, internet_addr);
+	if (CtdlDirectoryAddUser(internet_addr, citadel_addr))
+	{
+		StrBufAppendPrintf(CCC->StatusMessage, "\n%d|", CIT_OK);
+		StrBufAppendBufPlain(CCC->StatusMessage, internet_addr, -1, 0);
+		StrBufAppendBufPlain(CCC->StatusMessage, HKEY("|"), 0);
+		StrBufAppendBufPlain(CCC->StatusMessage, _("Successfully added email address."), -1, 0);
+		return 1;
+	}
+	else
+	{
+		StrBufAppendPrintf(CCC->StatusMessage, "\n%d|", ERROR+ ILLEGAL_VALUE);
+		StrBufAppendBufPlain(CCC->StatusMessage, internet_addr, -1, 0);
+		StrBufAppendBufPlain(CCC->StatusMessage, HKEY("|"), 0);
+		StrBufAppendBufPlain(CCC->StatusMessage, _("Unable to add this email address. It does not match any local domain."), -1, 0);
+		return 0;
+	}
 }
 
 
@@ -198,6 +214,7 @@ void cmd_igab(char *argbuf) {
 	CtdlDirectoryInit();
 
 	/* We want *all* vCards in this room */
+	NewStrBufDupAppendFlush(&CC->StatusMessage, NULL, NULL, 0);
 	CtdlForEachMessage(MSGS_ALL, 0, NULL, "[Tt][Ee][Xx][Tt]/.*[Vv][Cc][Aa][Rr][Dd]$",
 		NULL, vcard_add_to_directory, NULL);
 
@@ -213,10 +230,14 @@ void cmd_igab(char *argbuf) {
  * Internet messages.  If there is, stick it in the buffer.
  */
 void extract_inet_email_addrs(char *emailaddrbuf, size_t emailaddrbuf_len,
-				char *secemailaddrbuf, size_t secemailaddrbuf_len,
-				struct vCard *v, int local_addrs_only) {
+			      char *secemailaddrbuf, size_t secemailaddrbuf_len,
+			      struct vCard *v,
+			      int local_addrs_only)
+{
+	struct CitContext *CCC = CC;		/* put this on the stack, just for speed */
 	char *s, *k, *addr;
 	int instance = 0;
+	int IsDirectoryAddress;
 	int saved_instance = 0;
 
 	/* Go through the vCard searching for *all* Internet email addresses
@@ -227,8 +248,9 @@ void extract_inet_email_addrs(char *emailaddrbuf, size_t emailaddrbuf_len,
 			addr = strdup(s);
 			striplt(addr);
 			if (!IsEmptyStr(addr)) {
-				if ( (IsDirectory(addr, 1)) || 
-			     	(!local_addrs_only) ) {
+				IsDirectoryAddress = IsDirectory(addr, 1);
+				if ( IsDirectoryAddress || !local_addrs_only)
+				{
 					++saved_instance;
 					if ((saved_instance == 1) && (emailaddrbuf != NULL)) {
 						safestrncpy(emailaddrbuf, addr, emailaddrbuf_len);
@@ -244,6 +266,13 @@ void extract_inet_email_addrs(char *emailaddrbuf, size_t emailaddrbuf_len,
 						}
 					}
 				}
+				if (!IsDirectoryAddress && local_addrs_only)
+				{
+					StrBufAppendPrintf(CCC->StatusMessage, "\n%d|", ERROR+ ILLEGAL_VALUE);
+					StrBufAppendBufPlain(CCC->StatusMessage, addr, -1, 0);
+					StrBufAppendBufPlain(CCC->StatusMessage, HKEY("|"), 0);
+					StrBufAppendBufPlain(CCC->StatusMessage, _("unable to add this emailaddress; its not matching our domain."), -1, 0);
+				}
 			}
 			free(addr);
 		}
@@ -284,7 +313,7 @@ void vcard_extract_vcard(char *name, char *filename, char *partnum, char *disp,
 	if (  (!strcasecmp(cbtype, "text/x-vcard"))
 	   || (!strcasecmp(cbtype, "text/vcard")) ) {
 
-		syslog(LOG_DEBUG, "Part %s contains a vCard!  Loading...\n", partnum);
+		syslog(LOG_DEBUG, "Part %s contains a vCard!  Loading...", partnum);
 		if (*v != NULL) {
 			vcard_free(*v);
 		}
@@ -509,6 +538,11 @@ int vcard_upload_aftersave(struct CtdlMessage *msg) {
 
 	ptr = msg->cm_fields['M'];
 	if (ptr == NULL) return(0);
+
+	NewStrBufDupAppendFlush(&CC->StatusMessage, NULL, NULL, 0);
+
+	StrBufPrintf(CC->StatusMessage, "%d\n", LISTING_FOLLOWS);
+
 	while (ptr != NULL) {
 	
 		linelen = strcspn(ptr, "\n");
@@ -809,7 +843,7 @@ void vcard_newuser(struct ctdluser *usbuf) {
 	struct vCard *v;
 
 	vcard_fn_to_n(vname, usbuf->fullname, sizeof vname);
-	syslog(LOG_DEBUG, "Converted <%s> to <%s>\n", usbuf->fullname, vname);
+	syslog(LOG_DEBUG, "Converted <%s> to <%s>", usbuf->fullname, vname);
 
 	/* Create and save the vCard */
 	v = vcard_new();
@@ -828,7 +862,7 @@ void vcard_newuser(struct ctdluser *usbuf) {
 		if (getpwuid_r(usbuf->uid, &pwd, pwd_buffer, sizeof pwd_buffer) != NULL) {
 #else // SOLARIS_GETPWUID
 		struct passwd *result = NULL;
-		syslog(LOG_DEBUG, "Searching for uid %d\n", usbuf->uid);
+		syslog(LOG_DEBUG, "Searching for uid %d", usbuf->uid);
 		if (getpwuid_r(usbuf->uid, &pwd, pwd_buffer, sizeof pwd_buffer, &result) == 0) {
 #endif // HAVE_GETPWUID_R
 			snprintf(buf, sizeof buf, "%s@%s", pwd.pw_name, config.c_fqdn);
@@ -1013,9 +1047,9 @@ void dvca_mime_callback(char *name, char *filename, char *partnum, char *disp,
 		char *cbid, void *cbuserdata) {
 
 	struct vCard *v;
-	char displayname[256];
+	char displayname[256] = "";
 	int displayname_len;
-	char emailaddr[256];
+	char emailaddr[256] = "";
 	int i;
 	int has_commas = 0;
 
@@ -1115,13 +1149,13 @@ void check_get(void) {
 	time(&CC->lastcmd);
 	memset(cmdbuf, 0, sizeof cmdbuf); /* Clear it, just in case */
 	if (client_getln(cmdbuf, sizeof cmdbuf) < 1) {
-		syslog(LOG_CRIT, "vcard client disconnected: ending session.\n");
+		syslog(LOG_CRIT, "vcard client disconnected: ending session.");
 		CC->kill_me = KILLME_CLIENT_DISCONNECTED;
 		return;
 	}
-	syslog(LOG_INFO, ": %s\n", cmdbuf);
+	syslog(LOG_INFO, ": %s", cmdbuf);
 	while (strlen(cmdbuf) < 3) strcat(cmdbuf, " ");
-	syslog(LOG_INFO, "[ %s]\n", cmdbuf);
+	syslog(LOG_INFO, "[ %s]", cmdbuf);
 	
 	if (strncasecmp(cmdbuf, "GET ", 4)==0)
 	{
@@ -1138,22 +1172,20 @@ void check_get(void) {
 		{
 
 			cprintf("200 OK %s\n", internet_addr);
-			syslog(LOG_INFO, "sending 200 OK for the room %s\n", rcpt->display_recp);
+			syslog(LOG_INFO, "sending 200 OK for the room %s", rcpt->display_recp);
 		}
 		else 
 		{
 			cprintf("500 REJECT noone here by that name.\n");
 			
-			syslog(LOG_INFO, "sending 500 REJECT noone here by that name: %s\n", internet_addr);
+			syslog(LOG_INFO, "sending 500 REJECT no one here by that name: %s", internet_addr);
 		}
 		if (rcpt != NULL) 
 			free_recipients(rcpt);
 	}
-	else 
-	{
+	else {
 		cprintf("500 REJECT invalid Query.\n");
-		
-		syslog(LOG_INFO, "sending 500 REJECT invalid Query: %s\n", internet_addr);
+		syslog(LOG_INFO, "sending 500 REJECT invalid query: %s", internet_addr);
 	}
 }
 
@@ -1175,7 +1207,7 @@ void vcard_CtdlCreateRoom(void)
 
 	/* Set expiration policy to manual; otherwise objects will be lost! */
 	if (CtdlGetRoomLock(&qr, USERCONTACTSROOM)) {
-		syslog(LOG_ERR, "Couldn't get the user CONTACTS room!\n");
+		syslog(LOG_ERR, "Couldn't get the user CONTACTS room!");
 		return;
 	}
 	qr.QRep.expire_mode = EXPIRE_MANUAL;
@@ -1349,7 +1381,7 @@ void store_this_ha(struct addresses_to_be_filed *aptr) {
 			}
 			vcard_free(v);
 
-			syslog(LOG_DEBUG, "Adding contact: %s\n", recipient);
+			syslog(LOG_DEBUG, "Adding contact: %s", recipient);
 			CtdlSubmitMsg(vmsg, NULL, aptr->roomname, QP_EADDR);
 			CtdlFreeMessage(vmsg);
 		}
@@ -1458,7 +1490,7 @@ CTDL_MODULE_INIT(vcard)
 			if (fp != NULL) fclose(fp);
 			rv = chown(filename, CTDLUID, (-1));
 			if (rv == -1)
-				syslog(LOG_EMERG, "Failed to adjust ownership of: %s [%s]\n", 
+				syslog(LOG_EMERG, "Failed to adjust ownership of: %s [%s]", 
 				       filename, strerror(errno));
 		}
 
