@@ -46,18 +46,156 @@
 #include "snprintf.h"
 #endif
 
-struct CleanupFunctionHook *CleanupHookTable = NULL;
-struct SessionFunctionHook *SessionHookTable = NULL;
-struct UserFunctionHook *UserHookTable = NULL;
-struct XmsgFunctionHook *XmsgHookTable = NULL;
-struct MessageFunctionHook *MessageHookTable = NULL;
-struct NetprocFunctionHook *NetprocHookTable = NULL;
-struct DeleteFunctionHook *DeleteHookTable = NULL;
-struct ServiceFunctionHook *ServiceHookTable = NULL;
-struct FixedOutputHook *FixedOutputTable = NULL;
-struct RoomFunctionHook *RoomHookTable = NULL;
-struct SearchFunctionHook *SearchFunctionHookTable = NULL;
+int DebugModules = 0;
+ 
+/*
+ * Structure defentitions for hook tables
+ */
 
+typedef struct __LogDebugEntry {
+	CtdlDbgFunction F;
+	const char *Name;
+	long Len;
+	const int *LogP;
+} LogDebugEntry;
+HashList *LogDebugEntryTable = NULL;
+
+typedef struct LogFunctionHook LogFunctionHook;
+struct LogFunctionHook {
+	LogFunctionHook *next;
+	int loglevel;
+	void (*h_function_pointer) (char *);
+};
+extern LogFunctionHook *LogHookTable;
+
+typedef struct FixedOutputHook FixedOutputHook;
+struct FixedOutputHook {
+	FixedOutputHook *next;
+	char content_type[64];
+	void (*h_function_pointer) (char *, int);
+};
+extern FixedOutputHook *FixedOutputTable;
+
+
+
+/*
+ * SessionFunctionHook extensions are used for any type of hook for which
+ * the context in which it's being called (which is determined by the event
+ * type) will make it obvious for the hook function to know where to look for
+ * pertinent data.
+ */
+typedef struct SessionFunctionHook SessionFunctionHook;
+struct SessionFunctionHook {
+	SessionFunctionHook *next;
+	void (*h_function_pointer) (void);
+	int eventtype;
+};
+extern SessionFunctionHook *SessionHookTable;
+
+
+/*
+ * UserFunctionHook extensions are used for any type of hook which implements
+ * an operation on a user or username (potentially) other than the one
+ * operating the current session.
+ */
+typedef struct UserFunctionHook UserFunctionHook;
+struct UserFunctionHook {
+	UserFunctionHook *next;
+	void (*h_function_pointer) (struct ctdluser *usbuf);
+	int eventtype;
+};
+extern UserFunctionHook *UserHookTable;
+
+/*
+ * MessageFunctionHook extensions are used for hooks which implement handlers
+ * for various types of message operations (save, read, etc.)
+ */
+typedef struct MessageFunctionHook MessageFunctionHook;
+struct MessageFunctionHook {
+	MessageFunctionHook *next;
+	int (*h_function_pointer) (struct CtdlMessage *msg);
+	int eventtype;
+};
+extern MessageFunctionHook *MessageHookTable;
+
+
+/*
+ * NetprocFunctionHook extensions are used for hooks which implement handlers
+ * for incoming network messages.
+ */
+typedef struct NetprocFunctionHook NetprocFunctionHook;
+struct NetprocFunctionHook {
+	NetprocFunctionHook *next;
+	int (*h_function_pointer) (struct CtdlMessage *msg, char *target_room);
+};
+extern NetprocFunctionHook *NetprocHookTable;
+
+
+/*
+ * DeleteFunctionHook extensions are used for hooks which get called when a
+ * message is about to be deleted.
+ */
+typedef struct DeleteFunctionHook DeleteFunctionHook;
+struct DeleteFunctionHook {
+	DeleteFunctionHook *next;
+	void (*h_function_pointer) (char *target_room, long msgnum);
+};
+extern DeleteFunctionHook *DeleteHookTable;
+
+
+/*
+ * ExpressMessageFunctionHook extensions are used for hooks which implement
+ * the sending of an instant message through various channels.  Any function
+ * registered should return the number of recipients to whom the message was
+ * successfully transmitted.
+ */
+typedef struct XmsgFunctionHook XmsgFunctionHook;
+struct XmsgFunctionHook {
+	XmsgFunctionHook *next;
+	int (*h_function_pointer) (char *, char *, char *, char *);
+	int order;
+};
+extern XmsgFunctionHook *XmsgHookTable;
+
+
+
+
+/*
+ * RoomFunctionHook extensions are used for hooks which impliment room
+ * processing functions when new messages are added EG. SIEVE.
+ */
+typedef struct RoomFunctionHook RoomFunctionHook;
+struct RoomFunctionHook {
+	RoomFunctionHook *next;
+	int (*fcn_ptr) (struct ctdlroom *);
+};
+extern RoomFunctionHook *RoomHookTable;
+
+
+
+typedef struct SearchFunctionHook SearchFunctionHook;
+struct SearchFunctionHook {
+	SearchFunctionHook *next;
+	void (*fcn_ptr) (int *, long **, const char *);
+	char *name;
+};
+extern SearchFunctionHook *SearchFunctionHookTable;
+
+
+CleanupFunctionHook *CleanupHookTable = NULL;
+CleanupFunctionHook *EVCleanupHookTable = NULL;
+SessionFunctionHook *SessionHookTable = NULL;
+UserFunctionHook *UserHookTable = NULL;
+XmsgFunctionHook *XmsgHookTable = NULL;
+MessageFunctionHook *MessageHookTable = NULL;
+NetprocFunctionHook *NetprocHookTable = NULL;
+DeleteFunctionHook *DeleteHookTable = NULL;
+ServiceFunctionHook *ServiceHookTable = NULL;
+FixedOutputHook *FixedOutputTable = NULL;
+RoomFunctionHook *RoomHookTable = NULL;
+SearchFunctionHook *SearchFunctionHookTable = NULL;
+
+typedef struct ProtoFunctionHook ProtoFunctionHook;
 struct ProtoFunctionHook {
 	void (*handler) (char *cmdbuf);
 	const char *cmd;
@@ -85,7 +223,7 @@ ConstStr ErrGeneral[] = {
 	{HKEY("To make both ways actualy take place restart the citserver with \"sendcommand down\"\n\n"
 	      "The errors returned by the system were:\n")},
 	{HKEY("You can recheck the above if you follow this faq item:\n"
-	      "http://www.citadel.org/doku.php/faq:mastering_your_os:net#netstat")}
+	      "http://www.citadel.org/doku.php?id=faq:mastering_your_os:net#netstat")}
 };
 
 ConstStr ErrPortShort = { HKEY("We couldn't bind all ports you configured to be provided by citadel server.\n")};
@@ -139,8 +277,8 @@ void LogPrintMessages(long err)
 	StrBufAppendBufPlain(Message, HKEY("\n\n"), 0);
 	StrBufAppendBufPlain(Message, CKEY(ErrGeneral[3]), 0);
 
-	syslog(LOG_EMERG, "%s", ChrPtr(Message));
-	syslog(LOG_EMERG, "%s", ErrSubject);
+	MOD_syslog(LOG_EMERG, "%s", ChrPtr(Message));
+	MOD_syslog(LOG_EMERG, "%s", ErrSubject);
 	quickie_message("Citadel", NULL, NULL, AIDEROOM, ChrPtr(Message), FMT_FIXED, ErrSubject);
 
 	FreeStrBuf(&Message);
@@ -171,10 +309,10 @@ void AddPortError(char *Port, char *ErrorMessage)
 int DLoader_Exec_Cmd(char *cmdbuf)
 {
 	void *vP;
-	struct ProtoFunctionHook *p;
+	ProtoFunctionHook *p;
 
 	if (GetHash(ProtoHookList, cmdbuf, 4, &vP) && (vP != NULL)) {
-		p = (struct ProtoFunctionHook*) vP;
+		p = (ProtoFunctionHook*) vP;
 		p->handler(&cmdbuf[5]);
 		return 1;
 	}
@@ -197,16 +335,127 @@ long FourHash(const char *key, long length)
 	return ret;
 }
 
+void CtdlRegisterDebugFlagHook(const char *Name, long Len, CtdlDbgFunction F, const int *LogP)
+{
+	LogDebugEntry *E;
+	if (LogDebugEntryTable == NULL)
+		LogDebugEntryTable = NewHash(1, NULL);
+	E = (LogDebugEntry*) malloc(sizeof(LogDebugEntry));
+	E->F = F;
+	E->Name = Name;
+	E->Len = Len;
+	E->LogP = LogP;
+	Put(LogDebugEntryTable, Name, Len, E, NULL);
+	
+}
+void CtdlSetDebugLogFacilities(const char **Str, long n)
+{
+	StrBuf *Token = NULL;
+	StrBuf *Buf = NULL;
+	const char *ch;
+	int i;
+	int DoAll = 0;
+	void *vptr;
+
+	for (i=0; i < n; i++){
+		if ((Str[i] != NULL) && !IsEmptyStr(Str[i])) {
+			if (strcmp(Str[i], "all") == 0) {
+				DoAll = 1;
+				continue;
+			}
+			Buf = NewStrBufPlain(Str[i], -1);
+			ch = NULL;
+			if (Token == NULL)
+				Token = NewStrBufPlain(NULL, StrLength(Buf));
+			while ((ch != StrBufNOTNULL) &&
+			       StrBufExtract_NextToken(Token, Buf, &ch, ',')) {
+				if (GetHash(LogDebugEntryTable, SKEY(Token), &vptr) && 
+				    (vptr != NULL))
+				{
+					LogDebugEntry *E = (LogDebugEntry*)vptr;
+					E->F(1);
+				}
+			}
+		}
+		FreeStrBuf(&Buf);
+	}
+	FreeStrBuf(&Token);
+	if (DoAll) {
+		long HKLen;
+		const char *ch;
+		HashPos *Pos;
+
+		Pos = GetNewHashPos(LogDebugEntryTable, 0);
+		while (GetNextHashPos(LogDebugEntryTable, Pos, &HKLen, &ch, &vptr)) {
+			LogDebugEntry *E = (LogDebugEntry*)vptr;
+			E->F(1);
+		}
+
+		DeleteHashPos(&Pos);
+	}
+}
+void cmd_log_get(char *argbuf)
+{
+	long HKLen;
+	const char *ch;
+	HashPos *Pos;
+	void *vptr;
+
+	if (CtdlAccessCheck(ac_aide)) return;
+
+	cprintf("%d Log modules enabled:\n", LISTING_FOLLOWS);
+
+	Pos = GetNewHashPos(LogDebugEntryTable, 0);
+
+	while (GetNextHashPos(LogDebugEntryTable, Pos, &HKLen, &ch, &vptr)) {
+		LogDebugEntry *E = (LogDebugEntry*)vptr;
+		cprintf("%s|%d\n", ch, *E->LogP);
+	}
+	
+	DeleteHashPos(&Pos);
+	cprintf("000\n");
+}
+void cmd_log_set(char *argbuf)
+{
+	void *vptr;
+	int lset;
+	int wlen;
+	char which[SIZ] = "";
+
+	if (CtdlAccessCheck(ac_aide)) return;
+
+	wlen = extract_token(which, argbuf, 0, '|', sizeof(which));
+	if (wlen < 0) wlen = 0;
+	lset = extract_int(argbuf, 1);
+	if (lset != 0) lset = 1;
+	if (GetHash(LogDebugEntryTable, which, wlen, &vptr) && 
+	    (vptr != NULL))
+	{
+		LogDebugEntry *E = (LogDebugEntry*)vptr;
+		E->F(lset);
+		cprintf("%d %s|%d\n", CIT_OK, which, lset);
+	}
+	else {
+		cprintf("%d Log setting %s not known\n", 
+			ERROR, which);
+	}
+}
+void CtdlDestroyDebugTable(void)
+{
+
+	DeleteHash(&LogDebugEntryTable);
+}
+
 void CtdlRegisterProtoHook(void (*handler) (char *), char *cmd, char *desc)
 {
-	struct ProtoFunctionHook *p;
+	ProtoFunctionHook *p;
 
 	if (ProtoHookList == NULL)
 		ProtoHookList = NewHash (1, FourHash);
 
 
-	p = (struct ProtoFunctionHook *)
-		malloc(sizeof(struct ProtoFunctionHook));
+	p = (ProtoFunctionHook *)
+		malloc(sizeof(ProtoFunctionHook));
 
 	if (p == NULL) {
 		fprintf(stderr, "can't malloc new ProtoFunctionHook\n");
@@ -217,7 +466,7 @@ void CtdlRegisterProtoHook(void (*handler) (char *), char *cmd, char *desc)
 	p->desc = desc;
 
 	Put(ProtoHookList, cmd, 4, p, NULL);
-	syslog(LOG_INFO, "Registered server command %s (%s)\n", cmd, desc);
+	MOD_syslog(LOG_DEBUG, "Registered server command %s (%s)\n", cmd, desc);
 }
 
 void CtdlDestroyProtoHooks(void)
@@ -230,27 +479,27 @@ void CtdlDestroyProtoHooks(void)
 void CtdlRegisterCleanupHook(void (*fcn_ptr) (void))
 {
 
-	struct CleanupFunctionHook *newfcn;
+	CleanupFunctionHook *newfcn;
 
-	newfcn = (struct CleanupFunctionHook *)
-	    malloc(sizeof(struct CleanupFunctionHook));
+	newfcn = (CleanupFunctionHook *)
+	    malloc(sizeof(CleanupFunctionHook));
 	newfcn->next = CleanupHookTable;
 	newfcn->h_function_pointer = fcn_ptr;
 	CleanupHookTable = newfcn;
 
-	syslog(LOG_INFO, "Registered a new cleanup function\n");
+	MODM_syslog(LOG_DEBUG, "Registered a new cleanup function\n");
 }
 
 
 void CtdlUnregisterCleanupHook(void (*fcn_ptr) (void))
 {
-	struct CleanupFunctionHook *cur, *p;
+	CleanupFunctionHook *cur, *p;
 
 	for (cur = CleanupHookTable; cur != NULL; cur = cur->next) {
 		/* This will also remove duplicates if any */
 		while (cur != NULL &&
 				fcn_ptr == cur->h_function_pointer) {
-			syslog(LOG_INFO, "Unregistered cleanup function\n");
+			MODM_syslog(LOG_DEBUG, "Unregistered cleanup function\n");
 			p = cur->next;
 			if (cur == CleanupHookTable) {
 				CleanupHookTable = p;
@@ -261,14 +510,15 @@ void CtdlUnregisterCleanupHook(void (*fcn_ptr) (void))
 	}
 }
 
+
 void CtdlDestroyCleanupHooks(void)
 {
-	struct CleanupFunctionHook *cur, *p;
+	CleanupFunctionHook *cur, *p;
 
 	cur = CleanupHookTable;
 	while (cur != NULL)
 	{
-		syslog(LOG_INFO, "Destroyed cleanup function\n");
+		MODM_syslog(LOG_DEBUG, "Destroyed cleanup function\n");
 		p = cur->next;
 		free(cur);
 		cur = p;
@@ -276,35 +526,85 @@ void CtdlDestroyCleanupHooks(void)
 	CleanupHookTable = NULL;
 }
 
+void CtdlRegisterEVCleanupHook(void (*fcn_ptr) (void))
+{
+
+	CleanupFunctionHook *newfcn;
+
+	newfcn = (CleanupFunctionHook *)
+	    malloc(sizeof(CleanupFunctionHook));
+	newfcn->next = EVCleanupHookTable;
+	newfcn->h_function_pointer = fcn_ptr;
+	EVCleanupHookTable = newfcn;
+
+	MODM_syslog(LOG_DEBUG, "Registered a new cleanup function\n");
+}
+
+
+void CtdlUnregisterEVCleanupHook(void (*fcn_ptr) (void))
+{
+	CleanupFunctionHook *cur, *p;
+
+	for (cur = EVCleanupHookTable; cur != NULL; cur = cur->next) {
+		/* This will also remove duplicates if any */
+		while (cur != NULL &&
+				fcn_ptr == cur->h_function_pointer) {
+			MODM_syslog(LOG_DEBUG, "Unregistered cleanup function\n");
+			p = cur->next;
+			if (cur == EVCleanupHookTable) {
+				EVCleanupHookTable = p;
+			}
+			free(cur);
+			cur = p;
+		}
+	}
+}
+
+
+void CtdlDestroyEVCleanupHooks(void)
+{
+	CleanupFunctionHook *cur, *p;
+
+	cur = EVCleanupHookTable;
+	while (cur != NULL)
+	{
+		MODM_syslog(LOG_DEBUG, "Destroyed cleanup function\n");
+		p = cur->next;
+		free(cur);
+		cur = p;
+	}
+	EVCleanupHookTable = NULL;
+}
+
 
 void CtdlRegisterSessionHook(void (*fcn_ptr) (void), int EventType)
 {
 
-	struct SessionFunctionHook *newfcn;
+	SessionFunctionHook *newfcn;
 
-	newfcn = (struct SessionFunctionHook *)
-	    malloc(sizeof(struct SessionFunctionHook));
+	newfcn = (SessionFunctionHook *)
+	    malloc(sizeof(SessionFunctionHook));
 	newfcn->next = SessionHookTable;
 	newfcn->h_function_pointer = fcn_ptr;
 	newfcn->eventtype = EventType;
 	SessionHookTable = newfcn;
 
-	syslog(LOG_INFO, "Registered a new session function (type %d)\n",
-		EventType);
+	MOD_syslog(LOG_DEBUG, "Registered a new session function (type %d)\n",
+		   EventType);
 }
 
 
 void CtdlUnregisterSessionHook(void (*fcn_ptr) (void), int EventType)
 {
-	struct SessionFunctionHook *cur, *p;
+	SessionFunctionHook *cur, *p;
 
 	for (cur = SessionHookTable; cur != NULL; cur = cur->next) {
 		/* This will also remove duplicates if any */
 		while (cur != NULL &&
 				fcn_ptr == cur->h_function_pointer &&
 				EventType == cur->eventtype) {
-			syslog(LOG_INFO, "Unregistered session function (type %d)\n",
-					EventType);
+			MOD_syslog(LOG_DEBUG, "Unregistered session function (type %d)\n",
+				   EventType);
 			p = cur->next;
 			if (cur == SessionHookTable) {
 				SessionHookTable = p;
@@ -317,12 +617,12 @@ void CtdlUnregisterSessionHook(void (*fcn_ptr) (void), int EventType)
 
 void CtdlDestroySessionHooks(void)
 {
-	struct SessionFunctionHook *cur, *p;
+	SessionFunctionHook *cur, *p;
 
 	cur = SessionHookTable;
 	while (cur != NULL)
 	{
-		syslog(LOG_INFO, "Destroyed session function\n");
+		MODM_syslog(LOG_DEBUG, "Destroyed session function\n");
 		p = cur->next;
 		free(cur);
 		cur = p;
@@ -331,34 +631,34 @@ void CtdlDestroySessionHooks(void)
 }
 
 
-void CtdlRegisterUserHook(void (*fcn_ptr) (struct ctdluser *), int EventType)
+void CtdlRegisterUserHook(void (*fcn_ptr) (ctdluser *), int EventType)
 {
 
-	struct UserFunctionHook *newfcn;
+	UserFunctionHook *newfcn;
 
-	newfcn = (struct UserFunctionHook *)
-	    malloc(sizeof(struct UserFunctionHook));
+	newfcn = (UserFunctionHook *)
+	    malloc(sizeof(UserFunctionHook));
 	newfcn->next = UserHookTable;
 	newfcn->h_function_pointer = fcn_ptr;
 	newfcn->eventtype = EventType;
 	UserHookTable = newfcn;
 
-	syslog(LOG_INFO, "Registered a new user function (type %d)\n",
-		EventType);
+	MOD_syslog(LOG_DEBUG, "Registered a new user function (type %d)\n",
+		   EventType);
 }
 
 
 void CtdlUnregisterUserHook(void (*fcn_ptr) (struct ctdluser *), int EventType)
 {
-	struct UserFunctionHook *cur, *p;
+	UserFunctionHook *cur, *p;
 
 	for (cur = UserHookTable; cur != NULL; cur = cur->next) {
 		/* This will also remove duplicates if any */
 		while (cur != NULL &&
 				fcn_ptr == cur->h_function_pointer &&
 				EventType == cur->eventtype) {
-			syslog(LOG_INFO, "Unregistered user function (type %d)\n",
-					EventType);
+			MOD_syslog(LOG_DEBUG, "Unregistered user function (type %d)\n",
+				   EventType);
 			p = cur->next;
 			if (cur == UserHookTable) {
 				UserHookTable = p;
@@ -371,12 +671,12 @@ void CtdlUnregisterUserHook(void (*fcn_ptr) (struct ctdluser *), int EventType)
 
 void CtdlDestroyUserHooks(void)
 {
-	struct UserFunctionHook *cur, *p;
+	UserFunctionHook *cur, *p;
 
 	cur = UserHookTable;
 	while (cur != NULL)
 	{
-		syslog(LOG_INFO, "Destroyed user function \n");
+		MODM_syslog(LOG_DEBUG, "Destroyed user function \n");
 		p = cur->next;
 		free(cur);
 		cur = p;
@@ -389,32 +689,32 @@ void CtdlRegisterMessageHook(int (*handler)(struct CtdlMessage *),
 				int EventType)
 {
 
-	struct MessageFunctionHook *newfcn;
+	MessageFunctionHook *newfcn;
 
-	newfcn = (struct MessageFunctionHook *)
-	    malloc(sizeof(struct MessageFunctionHook));
+	newfcn = (MessageFunctionHook *)
+	    malloc(sizeof(MessageFunctionHook));
 	newfcn->next = MessageHookTable;
 	newfcn->h_function_pointer = handler;
 	newfcn->eventtype = EventType;
 	MessageHookTable = newfcn;
 
-	syslog(LOG_INFO, "Registered a new message function (type %d)\n",
-		EventType);
+	MOD_syslog(LOG_DEBUG, "Registered a new message function (type %d)\n",
+		   EventType);
 }
 
 
 void CtdlUnregisterMessageHook(int (*handler)(struct CtdlMessage *),
 		int EventType)
 {
-	struct MessageFunctionHook *cur, *p;
+	MessageFunctionHook *cur, *p;
 
 	for (cur = MessageHookTable; cur != NULL; cur = cur->next) {
 		/* This will also remove duplicates if any */
 		while (cur != NULL &&
 				handler == cur->h_function_pointer &&
 				EventType == cur->eventtype) {
-			syslog(LOG_INFO, "Unregistered message function (type %d)\n",
-					EventType);
+			MOD_syslog(LOG_DEBUG, "Unregistered message function (type %d)\n",
+				   EventType);
 			p = cur->next;
 			if (cur == MessageHookTable) {
 				MessageHookTable = p;
@@ -427,12 +727,12 @@ void CtdlUnregisterMessageHook(int (*handler)(struct CtdlMessage *),
 
 void CtdlDestroyMessageHook(void)
 {
-	struct MessageFunctionHook *cur, *p;
+	MessageFunctionHook *cur, *p;
 
 	cur = MessageHookTable; 
 	while (cur != NULL)
 	{
-		syslog(LOG_INFO, "Destroyed message function (type %d)\n", cur->eventtype);
+		MOD_syslog(LOG_DEBUG, "Destroyed message function (type %d)\n", cur->eventtype);
 		p = cur->next;
 		free(cur);
 		cur = p;
@@ -443,25 +743,25 @@ void CtdlDestroyMessageHook(void)
 
 void CtdlRegisterRoomHook(int (*fcn_ptr)(struct ctdlroom *))
 {
-	struct RoomFunctionHook *newfcn;
+	RoomFunctionHook *newfcn;
 
-	newfcn = (struct RoomFunctionHook *)
-	    malloc(sizeof(struct RoomFunctionHook));
+	newfcn = (RoomFunctionHook *)
+	    malloc(sizeof(RoomFunctionHook));
 	newfcn->next = RoomHookTable;
 	newfcn->fcn_ptr = fcn_ptr;
 	RoomHookTable = newfcn;
 
-	syslog(LOG_INFO, "Registered a new room function\n");
+	MODM_syslog(LOG_DEBUG, "Registered a new room function\n");
 }
 
 
 void CtdlUnregisterRoomHook(int (*fcn_ptr)(struct ctdlroom *))
 {
-	struct RoomFunctionHook *cur, *p;
+	RoomFunctionHook *cur, *p;
 
 	for (cur = RoomHookTable; cur != NULL; cur = cur->next) {
 		while (cur != NULL && fcn_ptr == cur->fcn_ptr) {
-			syslog(LOG_INFO, "Unregistered room function\n");
+			MODM_syslog(LOG_DEBUG, "Unregistered room function\n");
 			p = cur->next;
 			if (cur == RoomHookTable) {
 				RoomHookTable = p;
@@ -475,12 +775,12 @@ void CtdlUnregisterRoomHook(int (*fcn_ptr)(struct ctdlroom *))
 
 void CtdlDestroyRoomHooks(void)
 {
-	struct RoomFunctionHook *cur, *p;
+	RoomFunctionHook *cur, *p;
 
 	cur = RoomHookTable;
 	while (cur != NULL)
 	{
-		syslog(LOG_INFO, "Destroyed room function\n");
+		MODM_syslog(LOG_DEBUG, "Destroyed room function\n");
 		p = cur->next;
 		free(cur);
 		cur = p;
@@ -490,27 +790,27 @@ void CtdlDestroyRoomHooks(void)
 
 void CtdlRegisterNetprocHook(int (*handler)(struct CtdlMessage *, char *) )
 {
-	struct NetprocFunctionHook *newfcn;
+	NetprocFunctionHook *newfcn;
 
-	newfcn = (struct NetprocFunctionHook *)
-	    malloc(sizeof(struct NetprocFunctionHook));
+	newfcn = (NetprocFunctionHook *)
+	    malloc(sizeof(NetprocFunctionHook));
 	newfcn->next = NetprocHookTable;
 	newfcn->h_function_pointer = handler;
 	NetprocHookTable = newfcn;
 
-	syslog(LOG_INFO, "Registered a new netproc function\n");
+	MODM_syslog(LOG_DEBUG, "Registered a new netproc function\n");
 }
 
 
 void CtdlUnregisterNetprocHook(int (*handler)(struct CtdlMessage *, char *) )
 {
-	struct NetprocFunctionHook *cur, *p;
+	NetprocFunctionHook *cur, *p;
 
 	for (cur = NetprocHookTable; cur != NULL; cur = cur->next) {
 		/* This will also remove duplicates if any */
 		while (cur != NULL &&
 				handler == cur->h_function_pointer ) {
-			syslog(LOG_INFO, "Unregistered netproc function\n");
+			MODM_syslog(LOG_DEBUG, "Unregistered netproc function\n");
 			p = cur->next;
 			if (cur == NetprocHookTable) {
 				NetprocHookTable = p;
@@ -523,12 +823,12 @@ void CtdlUnregisterNetprocHook(int (*handler)(struct CtdlMessage *, char *) )
 
 void CtdlDestroyNetprocHooks(void)
 {
-	struct NetprocFunctionHook *cur, *p;
+	NetprocFunctionHook *cur, *p;
 
 	cur = NetprocHookTable;
 	while (cur != NULL)
 	{
-		syslog(LOG_INFO, "Destroyed netproc function\n");
+		MODM_syslog(LOG_DEBUG, "Destroyed netproc function\n");
 		p = cur->next;
 		free(cur);
 		cur = p;
@@ -539,27 +839,27 @@ void CtdlDestroyNetprocHooks(void)
 
 void CtdlRegisterDeleteHook(void (*handler)(char *, long) )
 {
-	struct DeleteFunctionHook *newfcn;
+	DeleteFunctionHook *newfcn;
 
-	newfcn = (struct DeleteFunctionHook *)
-	    malloc(sizeof(struct DeleteFunctionHook));
+	newfcn = (DeleteFunctionHook *)
+	    malloc(sizeof(DeleteFunctionHook));
 	newfcn->next = DeleteHookTable;
 	newfcn->h_function_pointer = handler;
 	DeleteHookTable = newfcn;
 
-	syslog(LOG_INFO, "Registered a new delete function\n");
+	MODM_syslog(LOG_DEBUG, "Registered a new delete function\n");
 }
 
 
 void CtdlUnregisterDeleteHook(void (*handler)(char *, long) )
 {
-	struct DeleteFunctionHook *cur, *p;
+	DeleteFunctionHook *cur, *p;
 
 	for (cur = DeleteHookTable; cur != NULL; cur = cur->next) {
 		/* This will also remove duplicates if any */
 		while (cur != NULL &&
 				handler == cur->h_function_pointer ) {
-			syslog(LOG_INFO, "Unregistered delete function\n");
+			MODM_syslog(LOG_DEBUG, "Unregistered delete function\n");
 			p = cur->next;
 			if (cur == DeleteHookTable) {
 				DeleteHookTable = p;
@@ -571,12 +871,12 @@ void CtdlUnregisterDeleteHook(void (*handler)(char *, long) )
 }
 void CtdlDestroyDeleteHooks(void)
 {
-	struct DeleteFunctionHook *cur, *p;
+	DeleteFunctionHook *cur, *p;
 
 	cur = DeleteHookTable;
 	while (cur != NULL)
 	{
-		syslog(LOG_INFO, "Destroyed delete function\n");
+		MODM_syslog(LOG_DEBUG, "Destroyed delete function\n");
 		p = cur->next;
 		free(cur);
 		cur = p;		
@@ -589,27 +889,27 @@ void CtdlDestroyDeleteHooks(void)
 
 void CtdlRegisterFixedOutputHook(char *content_type, void (*handler)(char *, int) )
 {
-	struct FixedOutputHook *newfcn;
+	FixedOutputHook *newfcn;
 
-	newfcn = (struct FixedOutputHook *)
-	    malloc(sizeof(struct FixedOutputHook));
+	newfcn = (FixedOutputHook *)
+	    malloc(sizeof(FixedOutputHook));
 	newfcn->next = FixedOutputTable;
 	newfcn->h_function_pointer = handler;
 	safestrncpy(newfcn->content_type, content_type, sizeof newfcn->content_type);
 	FixedOutputTable = newfcn;
 
-	syslog(LOG_INFO, "Registered a new fixed output function for %s\n", newfcn->content_type);
+	MOD_syslog(LOG_DEBUG, "Registered a new fixed output function for %s\n", newfcn->content_type);
 }
 
 
 void CtdlUnregisterFixedOutputHook(char *content_type)
 {
-	struct FixedOutputHook *cur, *p;
+	FixedOutputHook *cur, *p;
 
 	for (cur = FixedOutputTable; cur != NULL; cur = cur->next) {
 		/* This will also remove duplicates if any */
 		while (cur != NULL && (!strcasecmp(content_type, cur->content_type))) {
-			syslog(LOG_INFO, "Unregistered fixed output function for %s\n", content_type);
+			MOD_syslog(LOG_DEBUG, "Unregistered fixed output function for %s\n", content_type);
 			p = cur->next;
 			if (cur == FixedOutputTable) {
 				FixedOutputTable = p;
@@ -622,12 +922,12 @@ void CtdlUnregisterFixedOutputHook(char *content_type)
 
 void CtdlDestroyFixedOutputHooks(void)
 {
-	struct FixedOutputHook *cur, *p;
+	FixedOutputHook *cur, *p;
 
 	cur = FixedOutputTable; 
 	while (cur != NULL)
 	{
-		syslog(LOG_INFO, "Destroyed fixed output function for %s\n", cur->content_type);
+		MOD_syslog(LOG_DEBUG, "Destroyed fixed output function for %s\n", cur->content_type);
 		p = cur->next;
 		free(cur);
 		cur = p;
@@ -639,7 +939,7 @@ void CtdlDestroyFixedOutputHooks(void)
 /* returns nonzero if we found a hook and used it */
 int PerformFixedOutputHooks(char *content_type, char *content, int content_length)
 {
-	struct FixedOutputHook *fcn;
+	FixedOutputHook *fcn;
 
 	for (fcn = FixedOutputTable; fcn != NULL; fcn = fcn->next) {
 		if (!strcasecmp(content_type, fcn->content_type)) {
@@ -657,28 +957,28 @@ int PerformFixedOutputHooks(char *content_type, char *content, int content_lengt
 void CtdlRegisterXmsgHook(int (*fcn_ptr) (char *, char *, char *, char *), int order)
 {
 
-	struct XmsgFunctionHook *newfcn;
+	XmsgFunctionHook *newfcn;
 
-	newfcn = (struct XmsgFunctionHook *) malloc(sizeof(struct XmsgFunctionHook));
+	newfcn = (XmsgFunctionHook *) malloc(sizeof(XmsgFunctionHook));
 	newfcn->next = XmsgHookTable;
 	newfcn->order = order;
 	newfcn->h_function_pointer = fcn_ptr;
 	XmsgHookTable = newfcn;
-	syslog(LOG_INFO, "Registered a new x-msg function (priority %d)\n", order);
+	MOD_syslog(LOG_DEBUG, "Registered a new x-msg function (priority %d)\n", order);
 }
 
 
 void CtdlUnregisterXmsgHook(int (*fcn_ptr) (char *, char *, char *, char *), int order)
 {
-	struct XmsgFunctionHook *cur, *p;
+	XmsgFunctionHook *cur, *p;
 
 	for (cur = XmsgHookTable; cur != NULL; cur = cur->next) {
 		/* This will also remove duplicates if any */
 		while (cur != NULL &&
 				fcn_ptr == cur->h_function_pointer &&
 				order == cur->order) {
-			syslog(LOG_INFO, "Unregistered x-msg function "
-					"(priority %d)\n", order);
+			MOD_syslog(LOG_DEBUG, "Unregistered x-msg function "
+				   "(priority %d)\n", order);
 			p = cur->next;
 			if (cur == XmsgHookTable) {
 				XmsgHookTable = p;
@@ -691,12 +991,12 @@ void CtdlUnregisterXmsgHook(int (*fcn_ptr) (char *, char *, char *, char *), int
 
 void CtdlDestroyXmsgHooks(void)
 {
-	struct XmsgFunctionHook *cur, *p;
+	XmsgFunctionHook *cur, *p;
 
 	cur = XmsgHookTable;
 	while (cur != NULL)
 	{
-		syslog(LOG_INFO, "Destroyed x-msg function "
+		MOD_syslog(LOG_DEBUG, "Destroyed x-msg function "
 			"(priority %d)\n", cur->order);
 		p = cur->next;
 			
@@ -714,12 +1014,12 @@ void CtdlRegisterServiceHook(int tcp_port,
 			     void (*h_async_function) (void),
 			     const char *ServiceName)
 {
-	struct ServiceFunctionHook *newfcn;
+	ServiceFunctionHook *newfcn;
 	char *message;
 	char error[SIZ];
 
 	strcpy(error, "");
-	newfcn = (struct ServiceFunctionHook *) malloc(sizeof(struct ServiceFunctionHook));
+	newfcn = (ServiceFunctionHook *) malloc(sizeof(ServiceFunctionHook));
 	message = (char*) malloc (SIZ + SIZ);
 	
 	newfcn->next = ServiceHookTable;
@@ -735,7 +1035,7 @@ void CtdlRegisterServiceHook(int tcp_port,
 		snprintf(message, SIZ, "Unix domain socket '%s': ", sockpath);
 	}
 	else if (tcp_port <= 0) {	/* port -1 to disable */
-		syslog(LOG_INFO, "Service %s has been manually disabled, skipping\n", ServiceName);
+		MOD_syslog(LOG_INFO, "Service %s has been manually disabled, skipping\n", ServiceName);
 		free (message);
 		free(newfcn);
 		return;
@@ -752,12 +1052,12 @@ void CtdlRegisterServiceHook(int tcp_port,
 	if (newfcn->msock > 0) {
 		ServiceHookTable = newfcn;
 		strcat(message, "registered.");
-		syslog(LOG_INFO, "%s\n", message);
+		MOD_syslog(LOG_INFO, "%s\n", message);
 	}
 	else {
 		AddPortError(message, error);
 		strcat(message, "FAILED.");
-		syslog(LOG_CRIT, "%s\n", message);
+		MOD_syslog(LOG_CRIT, "%s\n", message);
 		free(newfcn);
 	}
 	free(message);
@@ -770,7 +1070,7 @@ void CtdlUnregisterServiceHook(int tcp_port, char *sockpath,
 			void (*h_async_function) (void)
 			)
 {
-	struct ServiceFunctionHook *cur, *p;
+	ServiceFunctionHook *cur, *p;
 
 	cur = ServiceHookTable;
 	while (cur != NULL) {
@@ -784,12 +1084,13 @@ void CtdlUnregisterServiceHook(int tcp_port, char *sockpath,
 				tcp_port == cur->tcp_port) {
 			close(cur->msock);
 			if (sockpath) {
-				syslog(LOG_INFO, "Closed UNIX domain socket %s\n",
-						sockpath);
+				MOD_syslog(LOG_INFO, "Closed UNIX domain socket %s\n",
+					   sockpath);
+				unlink(sockpath);
 			} else if (tcp_port) {
-				syslog(LOG_INFO, "Closed TCP port %d\n", tcp_port);
+				MOD_syslog(LOG_INFO, "Closed TCP port %d\n", tcp_port);
 			} else {
-				syslog(LOG_INFO, "Unregistered service \"%s\"\n", cur->ServiceName);
+				MOD_syslog(LOG_INFO, "Unregistered service \"%s\"\n", cur->ServiceName);
 			}
 			p = cur->next;
 			if (cur == ServiceHookTable) {
@@ -805,7 +1106,7 @@ void CtdlUnregisterServiceHook(int tcp_port, char *sockpath,
 void CtdlShutdownServiceHooks(void)
 {
 	/* sort of a duplicate of close_masters() but called earlier */
-	struct ServiceFunctionHook *cur;
+	ServiceFunctionHook *cur;
 
 	cur = ServiceHookTable;
 	while (cur != NULL) 
@@ -815,12 +1116,13 @@ void CtdlShutdownServiceHooks(void)
 			close(cur->msock);
 			cur->msock = -1;
 			if (cur->sockpath != NULL){
-				syslog(LOG_INFO, "[%s] Closed UNIX domain socket %s\n",
-					      cur->ServiceName,
-					      cur->sockpath);
+				MOD_syslog(LOG_INFO, "[%s] Closed UNIX domain socket %s\n",
+					   cur->ServiceName,
+					   cur->sockpath);
+				unlink(cur->sockpath);
 			} else {
-				syslog(LOG_INFO, "[%s] closing service\n", 
-					      cur->ServiceName);
+				MOD_syslog(LOG_INFO, "[%s] closing service\n", 
+					   cur->ServiceName);
 			}
 		}
 		cur = cur->next;
@@ -829,19 +1131,20 @@ void CtdlShutdownServiceHooks(void)
 
 void CtdlDestroyServiceHook(void)
 {
-	struct ServiceFunctionHook *cur, *p;
+	ServiceFunctionHook *cur, *p;
 
 	cur = ServiceHookTable;
 	while (cur != NULL)
 	{
 		close(cur->msock);
 		if (cur->sockpath) {
-			syslog(LOG_INFO, "Closed UNIX domain socket %s\n",
-				cur->sockpath);
+			MOD_syslog(LOG_INFO, "Closed UNIX domain socket %s\n",
+				   cur->sockpath);
+			unlink(cur->sockpath);
 		} else if (cur->tcp_port) {
-			syslog(LOG_INFO, "Closed TCP port %d\n", cur->tcp_port);
+			MOD_syslog(LOG_INFO, "Closed TCP port %d\n", cur->tcp_port);
 		} else {
-			syslog(LOG_INFO, "Destroyed service \"%s\"\n", cur->ServiceName);
+			MOD_syslog(LOG_INFO, "Destroyed service \"%s\"\n", cur->ServiceName);
 		}
 		p = cur->next;
 		free(cur);
@@ -852,29 +1155,29 @@ void CtdlDestroyServiceHook(void)
 
 void CtdlRegisterSearchFuncHook(void (*fcn_ptr)(int *, long **, const char *), char *name)
 {
-	struct SearchFunctionHook *newfcn;
+	SearchFunctionHook *newfcn;
 
 	if (!name || !fcn_ptr) {
 		return;
 	}
 	
-	newfcn = (struct SearchFunctionHook *)
-	    malloc(sizeof(struct SearchFunctionHook));
+	newfcn = (SearchFunctionHook *)
+	    malloc(sizeof(SearchFunctionHook));
 	newfcn->next = SearchFunctionHookTable;
 	newfcn->name = name;
 	newfcn->fcn_ptr = fcn_ptr;
 	SearchFunctionHookTable = newfcn;
 
-	syslog(LOG_INFO, "Registered a new search function (%s)\n", name);
+	MOD_syslog(LOG_DEBUG, "Registered a new search function (%s)\n", name);
 }
 
 void CtdlUnregisterSearchFuncHook(void (*fcn_ptr)(int *, long **, const char *), char *name)
 {
-	struct SearchFunctionHook *cur, *p;
+	SearchFunctionHook *cur, *p;
 	
 	for (cur = SearchFunctionHookTable; cur != NULL; cur = cur->next) {
 		while (fcn_ptr && (cur->fcn_ptr == fcn_ptr) && name && !strcmp(name, cur->name)) {
-			syslog(LOG_INFO, "Unregistered search function(%s)\n", name);
+			MOD_syslog(LOG_DEBUG, "Unregistered search function(%s)\n", name);
 			p = cur->next;
 			if (cur == SearchFunctionHookTable) {
 				SearchFunctionHookTable = p;
@@ -887,7 +1190,7 @@ void CtdlUnregisterSearchFuncHook(void (*fcn_ptr)(int *, long **, const char *),
 
 void CtdlDestroySearchHooks(void)
 {
-        struct SearchFunctionHook *cur, *p;
+        SearchFunctionHook *cur, *p;
 
 	cur = SearchFunctionHookTable;
 	SearchFunctionHookTable = NULL;
@@ -900,7 +1203,7 @@ void CtdlDestroySearchHooks(void)
 
 void CtdlModuleDoSearch(int *num_msgs, long **search_msgs, const char *search_string, const char *func_name)
 {
-	struct SearchFunctionHook *fcn = NULL;
+	SearchFunctionHook *fcn = NULL;
 
 	for (fcn = SearchFunctionHookTable; fcn != NULL; fcn = fcn->next) {
 		if (!func_name || !strcmp(func_name, fcn->name)) {
@@ -914,7 +1217,7 @@ void CtdlModuleDoSearch(int *num_msgs, long **search_msgs, const char *search_st
 
 void PerformSessionHooks(int EventType)
 {
-	struct SessionFunctionHook *fcn = NULL;
+	SessionFunctionHook *fcn = NULL;
 
 	for (fcn = SessionHookTable; fcn != NULL; fcn = fcn->next) {
 		if (fcn->eventtype == EventType) {
@@ -926,9 +1229,9 @@ void PerformSessionHooks(int EventType)
 	}
 }
 
-void PerformUserHooks(struct ctdluser *usbuf, int EventType)
+void PerformUserHooks(ctdluser *usbuf, int EventType)
 {
-	struct UserFunctionHook *fcn = NULL;
+	UserFunctionHook *fcn = NULL;
 
 	for (fcn = UserHookTable; fcn != NULL; fcn = fcn->next) {
 		if (fcn->eventtype == EventType) {
@@ -939,15 +1242,15 @@ void PerformUserHooks(struct ctdluser *usbuf, int EventType)
 
 int PerformMessageHooks(struct CtdlMessage *msg, int EventType)
 {
-	struct MessageFunctionHook *fcn = NULL;
+	MessageFunctionHook *fcn = NULL;
 	int total_retval = 0;
 
 	/* Other code may elect to protect this message from server-side
 	 * handlers; if this is the case, don't do anything.
-	syslog(LOG_DEBUG, "** Event type is %d, flags are %d\n", EventType, msg->cm_flags);
+	MOD_syslog(LOG_DEBUG, "** Event type is %d, flags are %d\n", EventType, msg->cm_flags);
 	 */
 	if (msg->cm_flags & CM_SKIP_HOOKS) {
-		syslog(LOG_DEBUG, "Skipping hooks\n");
+		MODM_syslog(LOG_DEBUG, "Skipping hooks\n");
 		return(0);
 	}
 
@@ -969,10 +1272,10 @@ int PerformMessageHooks(struct CtdlMessage *msg, int EventType)
 
 int PerformRoomHooks(struct ctdlroom *target_room)
 {
-	struct RoomFunctionHook *fcn;
+	RoomFunctionHook *fcn;
 	int total_retval = 0;
 
-	syslog(LOG_DEBUG, "Performing room hooks for <%s>\n", target_room->QRname);
+	MOD_syslog(LOG_DEBUG, "Performing room hooks for <%s>\n", target_room->QRname);
 
 	for (fcn = RoomHookTable; fcn != NULL; fcn = fcn->next) {
 		total_retval = total_retval + (*fcn->fcn_ptr) (target_room);
@@ -986,7 +1289,7 @@ int PerformRoomHooks(struct ctdlroom *target_room)
 
 int PerformNetprocHooks(struct CtdlMessage *msg, char *target_room)
 {
-	struct NetprocFunctionHook *fcn;
+	NetprocFunctionHook *fcn;
 	int total_retval = 0;
 
 	for (fcn = NetprocHookTable; fcn != NULL; fcn = fcn->next) {
@@ -1003,7 +1306,7 @@ int PerformNetprocHooks(struct CtdlMessage *msg, char *target_room)
 
 void PerformDeleteHooks(char *room, long msgnum)
 {
-	struct DeleteFunctionHook *fcn;
+	DeleteFunctionHook *fcn;
 
 	for (fcn = DeleteHookTable; fcn != NULL; fcn = fcn->next) {
 		(*fcn->h_function_pointer) (room, msgnum);
@@ -1016,7 +1319,7 @@ void PerformDeleteHooks(char *room, long msgnum)
 
 int PerformXmsgHooks(char *sender, char *sender_email, char *recp, char *msg)
 {
-	struct XmsgFunctionHook *fcn;
+	XmsgFunctionHook *fcn;
 	int total_sent = 0;
 	int p;
 
@@ -1048,3 +1351,18 @@ void CtdlModuleStartCryptoMsgs(char *ok_response, char *nosup_response, char *er
 	CtdlStartTLS (ok_response, nosup_response, error_response);
 #endif
 }
+
+void DebugModulesEnable(const int n)
+{
+	DebugModules = n;
+}
+CTDL_MODULE_INIT(modules)
+{
+	if (!threading) {
+		CtdlRegisterDebugFlagHook(HKEY("modules"), DebugModulesEnable, &DebugModules);
+
+		CtdlRegisterProtoHook(cmd_log_get, "LOGP", "Print Log-parameters");
+		CtdlRegisterProtoHook(cmd_log_set, "LOGS", "Set Log-parameters");
+	}
+	return "modules";
+}
