@@ -100,6 +100,7 @@ int main(int argc, char **argv)
 	char relhome[PATH_MAX]="";
 	char ctdldir[PATH_MAX]=CTDLDIR;
 	int syslog_facility = LOG_DAEMON;
+	const char *eDebuglist[] = {NULL, NULL};
 #ifdef HAVE_RUN_DIR
 	struct stat filestats;
 #endif
@@ -113,59 +114,56 @@ int main(int argc, char **argv)
 	InitializeMasterTSD();
 
 	/* parse command-line arguments */
-	for (a=1; a<argc; ++a) {
+	while ((a=getopt(argc, argv, "l:dh:x:t:Dr")) != EOF) switch(a) {
 
-		if (!strncmp(argv[a], "-l", 2)) {
-			safestrncpy(facility, &argv[a][2], sizeof(facility));
+		case 'l':
+			safestrncpy(facility, optarg, sizeof(facility));
 			syslog_facility = SyslogFacility(facility);
-		}
+			break;
 
 		/* run in the background if -d was specified */
-		else if (!strcmp(argv[a], "-d")) {
+		case 'd':
 			running_as_daemon = 1;
-		}
+			break;
 
-		else if (!strncmp(argv[a], "-h", 2)) {
-			relh=argv[a][2]!='/';
+		case 'h':
+			relh = optarg[0] != '/';
 			if (!relh) {
-				safestrncpy(ctdl_home_directory, &argv[a][2], sizeof ctdl_home_directory);
+				safestrncpy(ctdl_home_directory, optarg, sizeof ctdl_home_directory);
 			}
 			else {
-				safestrncpy(relhome, &argv[a][2], sizeof relhome);
+				safestrncpy(relhome, optarg, sizeof relhome);
 			}
 			home=1;
-		}
+			break;
 
-		else if (!strncmp(argv[a], "-x", 2)) {
-			/* deprecated */
-		}
+		case 'x':
+			eDebuglist [0] = optarg;
+			break;
 
-		else if (!strncmp(argv[a], "-t", 2)) {
-			/* deprecated */
-		}
+		case 't':	/* deprecated */
+			break;
 
-		else if (!strncmp(argv[a], "-D", 2)) {
+		case 'D':
 			dbg = 1;
-		}
+			break;
 
-		/* -r tells the server not to drop root permissions. don't use
-		 * this unless you know what you're doing. this should be
-		 * removed in the next release if it proves unnecessary. */
-		else if (!strcmp(argv[a], "-r")) {
+		/* -r tells the server not to drop root permissions.
+		 * Don't use this unless you know what you're doing.
+		 */
+		case 'r':
 			drop_root_perms = 0;
-		}
+			break;
 
+		default:
 		/* any other parameter makes it crash and burn */
-		else {
 			fprintf(stderr,	"citserver: usage: "
 					"citserver "
-					"[-lLogFacility] "
-					"[-d] [-D] [-s] "
-					"[-hHomeDir]\n"
+					"[-l LogFacility] "
+					"[-d] [-D] [-r] "
+					"[-h HomeDir]\n"
 			);
 			exit(1);
-		}
-
 	}
 
 	openlog("citserver",
@@ -209,10 +207,9 @@ int main(int argc, char **argv)
 	syslog(LOG_DEBUG, "Called as: %s", argv[0]);
 	syslog(LOG_INFO, "%s", libcitadel_version_string());
 
-	/* Load site-specific parameters, and set the ipgm secret */
+	/* Load site-specific configuration */
 	syslog(LOG_INFO, "Loading citadel.config");
 	get_config();
-	config.c_ipgm_secret = rand();
 
 	/* get_control() MUST MUST MUST be called BEFORE the databases are opened!! */
 	syslog(LOG_INFO, "Acquiring control record");
@@ -279,7 +276,7 @@ int main(int argc, char **argv)
 	}
 	
 	/*
-	 * Bind the server to a Unix-domain socket.
+	 * Bind the server to a Unix-domain socket (user client access)
 	 */
 	CtdlRegisterServiceHook(0,
 				file_citadel_socket,
@@ -288,6 +285,17 @@ int main(int argc, char **argv)
 				do_async_loop,
 				CitadelServiceUDS);
 
+	/*
+	 * Bind the server to a Unix-domain socket (admin client access)
+	 */
+	CtdlRegisterServiceHook(0,
+				file_citadel_admin_socket,
+				citproto_begin_admin_session,
+				do_command_loop,
+				do_async_loop,
+				CitadelServiceUDS);
+	chmod(file_citadel_admin_socket, S_IRWXU);	/* for your eyes only */
+
 	/*
 	 * Bind the server to our favorite TCP port (usually 504).
 	 */
@@ -308,6 +316,9 @@ int main(int argc, char **argv)
 	
 	initialise_modules(0);
 
+	eDebuglist[1] = getenv("CITADEL_LOGDEBUG");
+	CtdlSetDebugLogFacilities(eDebuglist, 2);
+
 	/*
 	 * If we need host auth, start our chkpwd daemon.
 	 */
