@@ -398,7 +398,7 @@ static void flush_client_inbuf(void)
 	CitContext *CCC=CC;
 
 	FlushStrBuf(CCC->ReadBuf);
-	CCC->Pos = NULL;
+	CCC->RecvBuf->ReadWritePointer = NULL;
 
 }
 */
@@ -411,8 +411,9 @@ void client_close(void) {
 	CitContext *CCC = CC;
 
 	if (!CCC) return;
-	if (CCC->client_socket < 0) return;
+	if (CCC->client_socket <= 0) return;
 	syslog(LOG_DEBUG, "Closing socket %d", CCC->client_socket);
+
 	close(CCC->client_socket);
 	CCC->client_socket = -1 ;
 }
@@ -447,13 +448,14 @@ int client_write(const char *buf, int nbytes)
 		snprintf(fn, SIZ, "/tmp/foolog_%s.%d", Ctx->ServiceName, Ctx->cs_pid);
 		
 		fd = fopen(fn, "a+");
-		fprintf(fd, "Sending: BufSize: %d BufContent: [",
-			nbytes);
-		rv = fwrite(buf, nbytes, 1, fd);
-		fprintf(fd, "]\n");
-		
-			
-		fclose(fd);
+		if (fd)
+		{
+		    fprintf(fd, "Sending: BufSize: %d BufContent: [",
+			    nbytes);
+		    rv = fwrite(buf, nbytes, 1, fd);
+		    fprintf(fd, "]\n");
+		    fclose(fd);
+		}
 	}
 #endif
 //	flush_client_inbuf();
@@ -616,8 +618,8 @@ int client_read_blob(StrBuf *Target, int bytes, int timeout)
 		fclose(fd);
 #endif
 		retval = StrBufReadBLOBBuffered(Target, 
-						CCC->ReadBuf,
-						&CCC->Pos,
+						CCC->RecvBuf.Buf,
+						&CCC->RecvBuf.ReadWritePointer,
 						&CCC->client_socket,
 						1, 
 						bytes,
@@ -649,8 +651,9 @@ int client_read_blob(StrBuf *Target, int bytes, int timeout)
  */
 void client_set_inbound_buf(long N)
 {
-	FlushStrBuf(CC->ReadBuf);
-	ReAdjustEmptyBuf(CC->ReadBuf, N * SIZ, N * SIZ);
+	CitContext *CCC=CC;
+	FlushStrBuf(CCC->RecvBuf.Buf);
+	ReAdjustEmptyBuf(CCC->RecvBuf.Buf, N * SIZ, N * SIZ);
 }
 
 int client_read_random_blob(StrBuf *Target, int timeout)
@@ -664,18 +667,18 @@ int client_read_random_blob(StrBuf *Target, int timeout)
 		long len;
 		const char *pch;
 		
-		len = StrLength(CCC->ReadBuf);
-		pch = ChrPtr(CCC->ReadBuf);
+		len = StrLength(CCC->RecvBuf.Buf);
+		pch = ChrPtr(CCC->RecvBuf.Buf);
 
 		if (len > 0)
 		{
-			if (CCC->Pos != NULL) {
-				len -= CCC->Pos - pch;
-				pch = CCC->Pos;
+			if (CCC->RecvBuf.ReadWritePointer != NULL) {
+				len -= CCC->RecvBuf.ReadWritePointer - pch;
+				pch = CCC->RecvBuf.ReadWritePointer;
 			}
 			StrBufAppendBufPlain(Target, pch, len, 0);
-			FlushStrBuf(CCC->ReadBuf);
-			CCC->Pos = NULL;
+			FlushStrBuf(CCC->RecvBuf.Buf);
+			CCC->RecvBuf.ReadWritePointer = NULL;
 #ifdef BIGBAD_IODBG
 			{
 				int rv = 0;
@@ -727,10 +730,10 @@ int client_read_to(char *buf, int bytes, int timeout)
 
 int HaveMoreLinesWaiting(CitContext *CCC)
 {
-	if ((CCC->kill_me != 0) || (
-	    (CCC->Pos == NULL) && 
-	    (StrLength(CCC->ReadBuf) == 0) && 
-	    (CCC->client_socket != -1)) )
+	if ((CCC->kill_me != 0) ||
+	    ( (CCC->RecvBuf.ReadWritePointer == NULL) && 
+	      (StrLength(CCC->RecvBuf.Buf) == 0) && 
+	      (CCC->client_socket != -1)) )
 		return 0;
 	else
 		return 1;
@@ -768,10 +771,10 @@ int CtdlClientGetLine(StrBuf *Target)
 		snprintf(fn, SIZ, "/tmp/foolog_%s.%d", CCC->ServiceName, CCC->cs_pid);
 
 		fd = fopen(fn, "a+");
-		pch = ChrPtr(CCC->ReadBuf);
-		len = StrLength(CCC->ReadBuf);
-		if (CCC->Pos != NULL)
-			rlen = CC->Pos - pch;
+		pch = ChrPtr(CCC->RecvBuf.Buf);
+		len = StrLength(CCC->RecvBuf.Buf);
+		if (CCC->RecvBuf.ReadWritePointer != NULL)
+			rlen = CCC->RecvBuf.ReadWritePointer - pch;
 		else
 			rlen = 0;
 
@@ -782,14 +785,14 @@ int CtdlClientGetLine(StrBuf *Target)
 			len, rlen);
 #endif
 		rc = client_readline_sslbuffer(Target,
-					       CCC->ReadBuf,
-					       &CCC->Pos,
+					       CCC->RecvBuf.Buf,
+					       &CCC->RecvBuf.ReadWritePointer,
 					       1);
 #ifdef BIGBAD_IODBG
-                pch = ChrPtr(CCC->ReadBuf);
-                nlen = StrLength(CCC->ReadBuf);
-                if (CCC->Pos != NULL)
-                        nrlen = CC->Pos - pch;
+                pch = ChrPtr(CCC->RecvBuf.Buf);
+                nlen = StrLength(CCC->RecvBuf.Buf);
+                if (CCC->RecvBuf.ReadWritePointer != NULL)
+                        nrlen = CCC->RecvBuf.ReadWritePointer - pch;
                 else
                         nrlen = 0;
 /*
@@ -821,10 +824,10 @@ int CtdlClientGetLine(StrBuf *Target)
 		snprintf(fn, SIZ, "/tmp/foolog_%s.%d", CCC->ServiceName, CCC->cs_pid);
 
 		fd = fopen(fn, "a+");
-		pch = ChrPtr(CCC->ReadBuf);
-		len = StrLength(CCC->ReadBuf);
-		if (CCC->Pos != NULL)
-			rlen = CC->Pos - pch;
+		pch = ChrPtr(CCC->RecvBuf.Buf);
+		len = StrLength(CCC->RecvBuf.Buf);
+		if (CCC->RecvBuf.ReadWritePointer != NULL)
+			rlen = CCC->RecvBuf.ReadWritePointer - pch;
 		else
 			rlen = 0;
 
@@ -835,18 +838,18 @@ int CtdlClientGetLine(StrBuf *Target)
 			len, rlen);
 #endif
 		rc = StrBufTCP_read_buffered_line_fast(Target, 
-						       CCC->ReadBuf,
-						       &CCC->Pos,
+						       CCC->RecvBuf.Buf,
+						       &CCC->RecvBuf.ReadWritePointer,
 						       &CCC->client_socket,
 						       5,
 						       1,
 						       &Error);
 
 #ifdef BIGBAD_IODBG
-                pch = ChrPtr(CCC->ReadBuf);
-                nlen = StrLength(CCC->ReadBuf);
-                if (CCC->Pos != NULL)
-                        nrlen = CC->Pos - pch;
+                pch = ChrPtr(CCC->RecvBuf.Buf);
+                nlen = StrLength(CCC->RecvBuf.Buf);
+                if (CCC->RecvBuf.ReadWritePointer != NULL)
+                        nrlen = CCC->RecvBuf.ReadWritePointer - pch;
                 else
                         nrlen = 0;
 /*
@@ -924,16 +927,18 @@ void close_masters (void)
 
 		if (serviceptr->tcp_port > 0)
 		{
-			syslog(LOG_INFO, "Closing listener on port %d\n",
-				serviceptr->tcp_port);
+			syslog(LOG_INFO, "Closing %d listener on port %d\n",
+			       serviceptr->msock,
+			       serviceptr->tcp_port);
 			serviceptr->tcp_port = 0;
 		}
 		
 		if (serviceptr->sockpath != NULL)
-			syslog(LOG_INFO, "Closing listener on '%s'\n",
-				serviceptr->sockpath);
-
-		close(serviceptr->msock);
+			syslog(LOG_INFO, "Closing %d listener on '%s'\n",
+			       serviceptr->msock,
+			       serviceptr->sockpath);
+                if (serviceptr->msock != -1)
+			close(serviceptr->msock);
 		/* If it's a Unix domain socket, remove the file. */
 		if (serviceptr->sockpath != NULL) {
 			unlink(serviceptr->sockpath);
@@ -967,6 +972,7 @@ void sysdep_master_cleanup(void) {
 	CtdlDestroyServiceHook();
 	CtdlDestroyRoomHooks();
 	CtdlDestroySearchHooks();
+	CtdlDestroyDebugTable();
 	#ifdef HAVE_BACKTRACE
 ///	eCrash_Uninit();
 	#endif
@@ -1090,7 +1096,7 @@ void checkcrash(void)
 			"factor.\n \n"
 			" You can obtain more information about this by enabling core dumps.\n \n"
 			" For more information, please see:\n \n"
-			" http://citadel.org/doku.php/faq:mastering_your_os:gdb#how.do.i.make.my.system.produce.core-files"
+			" http://citadel.org/doku.php?id=faq:mastering_your_os:gdb#how.do.i.make.my.system.produce.core-files"
 			"\n \n"
 
 			" If you have already done this, the core dump is likely to be found at %score.%d\n"
@@ -1167,10 +1173,13 @@ do_select:	force_purge = 0;
 		/* Next, add all of the client sockets. */
 		begin_critical_section(S_SESSION_TABLE);
 		for (ptr = ContextList; ptr != NULL; ptr = ptr->next) {
+			if ((ptr->state == CON_SYS) && (ptr->client_socket == 0))
+			    continue;
+
 			/* Don't select on dead sessions, only truly idle ones */
 			if (	(ptr->state == CON_IDLE)
 				&& (ptr->kill_me == 0)
-				&& (ptr->client_socket != -1)
+				&& (ptr->client_socket > 0)
 			) {
 				FD_SET(ptr->client_socket, &readfds);
 				if (ptr->client_socket > highest)
@@ -1221,7 +1230,9 @@ do_select:	force_purge = 0;
 				server_shutting_down = 1;
 				continue;
 			} else {
+#if 0
 				syslog(LOG_DEBUG, "Interrupted select()\n");
+#endif
 				if (server_shutting_down) {
 					--num_workers;
 					return(NULL);
@@ -1261,6 +1272,7 @@ do_select:	force_purge = 0;
 					con = CreateNewContext();
 
 					/* Assign our new socket number to it. */
+					con->tcp_port = serviceptr->tcp_port;
 					con->client_socket = ssock;
 					con->h_command_function = serviceptr->h_command_function;
 					con->h_async_function = serviceptr->h_async_function;
@@ -1361,13 +1373,6 @@ SKIP_SELECT:
 
 
 
-
-
-
-
-
-
-
 /*
  * SyslogFacility()
  * Translate text facility name to syslog.h defined value.
