@@ -1,21 +1,15 @@
 /*
  * Main source module for the client program.
  *
- * Copyright (c) 1987-2011 by the citadel.org team
+ * Copyright (c) 1987-2012 by the citadel.org team
  *
  *  This program is open source software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 3 of the License, or
- *  (at your option) any later version.
+ *  it under the terms of the GNU General Public License version 3.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 #include "sysdep.h"
@@ -82,7 +76,6 @@ extern char *moreprompt;
 char temp[PATH_MAX];		/* Name of general-purpose temp file */
 char temp2[PATH_MAX];		/* Name of general-purpose temp file */
 char tempdir[PATH_MAX];		/* Name of general-purpose temp directory */
-char editor_paths[MAX_EDITORS][SIZ];	/* paths to external editors */
 char printcmd[SIZ];		/* print command */
 int editor_pid = (-1);
 char fullname[USERNAME_SIZE];
@@ -858,10 +851,6 @@ void  gotoroomstep(CtdlIPC *ipc, int direction, int mode)
 
 	if (mode == 0) { /* not skipping */
 	    updatels(ipc);
-	} else {
-		if (rc_alt_semantics) {
-	    	updatelsa(ipc);
-		}
 	}
 
 	/* Free the room list */
@@ -978,7 +967,7 @@ void read_config(CtdlIPC *ipc)
 	    scr_printf("Enable color support: ");                              color(BRIGHT_CYAN); scr_printf("%s\n", (user->flags & US_COLOR)? "Yes" : "No");	     color(DIM_WHITE);
 	}
 	scr_printf("Be unlisted in userlog: ");                                color(BRIGHT_CYAN); scr_printf("%s\n", (user->flags & US_UNLISTED)? "Yes" : "No");    color(DIM_WHITE);
-	if (!IsEmptyStr(editor_paths[0])) {
+	if (!IsEmptyStr(editor_path)) {
     	scr_printf("Always enter messages with the full-screen editor: "); color(BRIGHT_CYAN); scr_printf("%s\n", (user->flags & US_EXTEDIT)? "Yes" : "No");     color(DIM_WHITE);
 	}
 	free(user);
@@ -1022,7 +1011,7 @@ void system_info(CtdlIPC *ipc)
     scr_printf("Connected users %d / Active users %d / Highest message #%ld\n", mrtg_users, mrtg_active_users, mrtg_himessage);
     scr_printf("Server uptime: %s\n", mrtg_server_uptime);
     scr_printf("Your system administrator is %s.\n", ipc->ServInfo.sysadm);
-    scr_printf("Copyright (C)1987-2011 by the Citadel development team\n");
+    scr_printf("Copyright (C)1987-2009 by the Citadel development team\n");
 }
 
 /*
@@ -1817,14 +1806,9 @@ NEWUSR:	if (IsEmptyStr(rc_password)) {
 				gotonext(ipc);
 				break;
 			case 47:			/* <A>bandon */
-				if (!rc_alt_semantics) {
-					updatelsa(ipc);
-				}
 				gotonext(ipc);
 				break;
 			case 90:			/* <.A>bandon */
-				if (!rc_alt_semantics)
-					updatelsa(ipc);
 				dotgoto(ipc, argbuf, 0, 0);
 				break;
 			case 58:			/* <M>ail */
@@ -1839,9 +1823,6 @@ NEWUSR:	if (IsEmptyStr(rc_password)) {
 				break;
 			case 52:
 				if (!IsEmptyStr(argbuf)) {
-					if (rc_alt_semantics) {
-						updatelsa(ipc);
-					}
 					dotgoto(ipc, argbuf, 0, 0);
 				}
 				break;
@@ -1939,9 +1920,7 @@ NEWUSR:	if (IsEmptyStr(rc_password)) {
 				break;
 			case 29:
 			case 30:
-				if (!rc_alt_semantics) {
-					updatels(ipc);
-				}
+				updatels(ipc);
 				termn8 = 1;
 				break;
 			case 48:
@@ -1999,8 +1978,7 @@ NEWUSR:	if (IsEmptyStr(rc_password)) {
 			case 15:
 				scr_printf("Are you sure (y/n)? ");
 				if (yesno() == 1) {
-					if (!rc_alt_semantics)
-						updatels(ipc);
+					updatels(ipc);
 					a = 0;
 					termn8 = 1;
 				}
@@ -2010,8 +1988,7 @@ NEWUSR:	if (IsEmptyStr(rc_password)) {
 				scr_printf("All users will be disconnected!  "
 					   "Really terminate the server? ");
 				if (yesno() == 1) {
-					if (!rc_alt_semantics)
-						updatels(ipc);
+					updatels(ipc);
 					r = CtdlIPCTerminateServerNow(ipc, aaa);
 					scr_printf("%s\n", aaa);
 					if (r / 100 == 2) {
@@ -2064,9 +2041,6 @@ NEWUSR:	if (IsEmptyStr(rc_password)) {
 				break;
 
 			case 6:
-				if (rc_alt_semantics) {
-					updatelsa(ipc);
-				}
 				gotonext(ipc);
 				break;
 
@@ -2094,10 +2068,6 @@ NEWUSR:	if (IsEmptyStr(rc_password)) {
 				do_internet_configuration(ipc);
 				break;
 
-			case 83:
-				check_message_base(ipc);
-				break;
-
 			case 84:
 				quiet_mode(ipc);
 				break;
@@ -2271,17 +2241,8 @@ NEWUSR:	if (IsEmptyStr(rc_password)) {
 				do_rssclient_configuration(ipc);
 				break;
 
-			default: /* allow some math on the command */
-				/* commands 100... to 100+MAX_EDITORS-1 will
-				   call the appropriate editor... in other
-				   words, command numbers 100+ will match
-				   the citadel.rc keys editor0, editor1, etc.*/
-				if (mcmd >= 100 && mcmd < (100+MAX_EDITORS))
-				{
-					/* entmsg mode >=2 select editor */
-					entmsg(ipc, 0, mcmd - 100 + 2, 0);
-					break;
-				}
+			default:
+				break;
 			}	/* end switch */
 	} while (termn8 == 0);
 
